USE tempdb
GO




CREATE TABLE temporal_test 
(
	i INT NOT NULL PRIMARY KEY, 
	j INT,
	start_datetime DATETIME2 GENERATED ALWAYS AS ROW START, 
	end_datetime DATETIME2 GENERATED ALWAYS AS ROW END,
	PERIOD FOR SYSTEM_TIME (start_datetime, end_datetime),
)
WITH (SYSTEM_VERSIONING = ON)
GO




INSERT temporal_test (i, j)
SELECT 1, 2
UNION ALL
SELECT 2, 3
GO




SELECT
	*
FROM temporal_test
GO




UPDATE temporal_test
SET
	j = 123
WHERE
	i = 1
GO

DELETE temporal_test
WHERE
	i = 2
GO




SELECT
	*
FROM temporal_test
GO




SELECT
	*
FROM temporal_test 
FOR SYSTEM_TIME BETWEEN '2016-01-01' AND '9999-12-31'
ORDER BY
	i,
	start_datetime
GO




--interesting aside
DROP TABLE temporal_test
GO




ALTER TABLE temporal_test SET (SYSTEM_VERSIONING = OFF)
GO


DROP TABLE temporal_test
GO




--Real timezone support


SELECT
	SYSDATETIMEOFFSET()
GO




SELECT
	*
FROM sys.time_zone_info
GO




SELECT
	SYSDATETIMEOFFSET() AT TIME ZONE 'Central Standard Time'
GO




--What time is it in London?
SELECT
	SYSDATETIMEOFFSET() AT TIME ZONE 'GMT Standard Time'
GO



--DST awareness
--On March 26 at 05:00 EST, what time was it in London?
SELECT
	CONVERT(DATETIME, '2016-03-26 05:00:00') AT TIME ZONE 'Eastern Standard Time' AT TIME ZONE 'GMT Standard Time'
GO


--And on the 27th?
SELECT
	CONVERT(DATETIME, '2016-03-27 05:00:00') AT TIME ZONE 'Eastern Standard Time' AT TIME ZONE 'GMT Standard Time'
GO




--best practice, use UTC for everything, always, and convert...

DECLARE @desired_time_zone VARCHAR(50) = 'Eastern Standard Time'

SELECT
	SYSUTCDATETIME() AT TIME ZONE 'UTC' AT TIME ZONE @desired_time_zone
GO


