USE tempdb
GO



--data compression can work quite well...
CREATE TABLE dbo.compress_me
(
	i INT IDENTITY(1,1) NOT NULL,
	j VARCHAR(MAX)
)
GO

INSERT dbo.compress_me (j)
SELECT TOP(100000)
	REPLICATE('a', 2500)
FROM
	master..spt_values AS a,
	master..spt_values AS b
GO

EXEC sp_spaceused 'dbo.compress_me'
GO

CREATE CLUSTERED INDEX i 
ON dbo.compress_me (i) 
WITH (DATA_COMPRESSION = PAGE)
GO

EXEC sp_spaceused 'dbo.compress_me'
GO




--try again with a slightly different setup...
DROP TABLE dbo.compress_me
GO




CREATE TABLE dbo.compress_me
(
	i INT IDENTITY(1,1) NOT NULL,
	j VARCHAR(MAX)
)
GO

INSERT dbo.compress_me (j)
SELECT TOP(25000)
	REPLICATE('a', 10000)
FROM
	master..spt_values AS a,
	master..spt_values AS b
GO

EXEC sp_spaceused 'dbo.compress_me'
GO

CREATE CLUSTERED INDEX i 
ON dbo.compress_me (i) 
WITH (DATA_COMPRESSION = PAGE)
GO

EXEC sp_spaceused 'dbo.compress_me'
GO




--solution? 
SELECT 
	DATALENGTH
	(
		--gzip!
		COMPRESS
		(
			REPLICATE(CONVERT(VARCHAR(MAX), 'xyz'), 100000)
		)
	)
GO




--add a column to hold the compressed data
ALTER TABLE dbo.compress_me
ADD j_compressed VARBINARY(MAX)
GO


--compress the data
UPDATE dbo.compress_me
SET j_compressed = COMPRESS(j)
GO


--get rid of the uncompressed data
ALTER TABLE dbo.compress_me
DROP COLUMN j
GO


--reclaim the space
ALTER INDEX i ON dbo.compress_me REBUILD
GO


--Looks nicer..!
EXEC sp_spaceused 'dbo.compress_me'
GO



--Kind of difficult to read
SELECT TOP(10)
	*
FROM dbo.compress_me
GO



--now expose the data...


--move the table to an internal version
EXEC sp_rename 'dbo.compress_me', 'compress_me_internal'
GO


--create a view...
CREATE VIEW dbo.compress_me
AS
	SELECT
		i,
		CONVERT(VARCHAR(MAX), DECOMPRESS(j_compressed)) AS j
	FROM dbo.compress_me_internal
GO



--better!
SELECT TOP(10) 
	* 
FROM dbo.compress_me
GO



--how about writes?
CREATE TRIGGER dbo.write_compress_me
ON dbo.compress_me
INSTEAD OF INSERT 
AS
	INSERT dbo.compress_me_internal 
	(
		j_compressed
	)
	SELECT
		COMPRESS(j)
	FROM inserted
GO


INSERT dbo.compress_me (j)
VALUES (REPLICATE(CONVERT(VARCHAR(MAX), 'compress me!'), 10000))
GO

